@@ -336,8 +336,12 @@ Bacon.hist <- function(d, set=get('info'), BCAD=set$BCAD, age.lab=c(), age.lim=c
     hsts <- list(); maxhist <- 0; minhist <- 1
     pb <- txtProgressBar(min=0, max=max(1,length(d)-1), style = 3)
     for(i in 1:length(d)) {
-      if(length(d) > 1)
-        setTxtProgressBar(pb, i)
+      if(length(d) > 1) {
+        if(length(d) < 500) # progress bar slows things down much if i is large
+          setTxtProgressBar(pb, i) else
+            if(i %% 10 == 0)
+              setTxtProgressBar(pb, i)
+      }
       ages <- Bacon.Age.d(d[i], set, BCAD=BCAD)
       if(length(ages[!is.na(ages)]) > 0) { # added !is.na(ages) 21 April 21
         hst <- density(ages)
@@ -389,6 +393,7 @@ Bacon.hist <- function(d, set=get('info'), BCAD=set$BCAD, age.lab=c(), age.lim=c
 }
 
 
+
 # to calculate age ranges
 Bacon.rng <- function(d, set=get('info'), BCAD=set$BCAD, prob=set$prob) {
   outfile <- paste0(set$prefix, ".out")
@@ -409,14 +414,22 @@ Bacon.rng <- function(d, set=get('info'), BCAD=set$BCAD, prob=set$prob) {
       rng[i,4] <- mean(ages)
     }
 
-  if(length(d) > 1)
-    setTxtProgressBar(pb, i)
+    if(length(d) > 1) {
+      if(length(d) < 500) # progress bar slows things down when i is large
+        setTxtProgressBar(pb, i) else
+          if(i %% 10 == 0)
+            setTxtProgressBar(pb, i)
+    }
+#  if(length(d) > 1)
+#    setTxtProgressBar(pb, i)
   }
   #if(length(d) > 1)
   #  close(pb)
   return(rng)
 }
 
+
+
 #' @name agemodel.it
 #' @title Extract one age-model iteration
 #' @description For one MCMC iteration (it), extract the corresponding age-depth model.
@@ -448,6 +461,7 @@ agemodel.it <- function(it, set=get('info'), BCAD=set$BCAD) {
 }
 
 
+
 # calculate slumpfree depths
 toslump <- function(d, slump, remove=FALSE) {
   d <- sort(d)
@@ -486,3 +500,59 @@ fromslump <- function(d, slump) {
   }
   return(dorig)
 }
+
+
+
+#' @name squeeze
+#' @title Squeeze some depths of a core
+#' @description Squeeze or compress depths below a boundary by a certain amount. Accompanies the stretch function; see the stretch function for code on running the accordion
+#' @param d The depth(s) to be squeezed
+#' @param boundary The depth below which depths should be squeezed
+#' @param times The factor by which the depths should be squeezed
+#' @author Maarten Blaauw
+#' @return The squeezed depth(s)
+#' @examples
+#'   squeeze(40,25,20)
+#' @export
+squeeze <- function(d, boundary, times) {
+  below <- which(d > boundary)
+  if(length(below) > 0)
+    d[below] <- boundary + (d[below]-boundary)/times
+  return(d)
+}
+
+
+
+#' @name stretch
+#' @title Stretch some depths of a core
+#' @description Stretch squeezed depths e.g., calculate the original depths of depths that were squeezed. Accompanies the squeeze function.
+#' @param d The depth(s) to be stretched
+#' @param boundary The depth below which depths should be stretched
+#' @param times The factor by which the depths should be stretched
+#' @author Maarten Blaauw
+#' @return The stretched depth(s)
+#' @examples
+#'   stretch(25.75,25,20)
+#' \dontrun{
+#'   # To play the accordion, first squeeze an existing core.
+#'   # Let's squeeze the depths below 10 cm core depth 20 times:
+#'   Bacon("accordion", 1)
+#'   dets <- info$dets
+#'   dets[,4] <- squeeze(dets[,4], 10, 20)
+#'
+#'   # make a new directory for the squeezed core, and place the dets file there:
+#'   dir.create("Bacon_runs/squeezed")
+#'   write.table(dets, "Bacon_runs/squeezed/squeezed.csv", row.names=FALSE, sep=",")
+#'
+#'   # now run that squeezed core, adding a boundary (10cm) and adapting the acc.mean prior (20x):
+#'   Bacon("squeezed", 1, boundary=10, acc.mean=c(5, 20*5))
+#'   # finally, plot while stretching the depths onto the original scale:
+#'   agedepth(accordion=c(10,20))
+#' }
+#' @export
+stretch <- function(d, boundary, times) {
+  below <- which(d > boundary)
+  if(length(below) > 0)
+    d[below] <- boundary + (d[below]-boundary)*times
+  return(d)
+}
