@@ -77,7 +77,7 @@ Input::Input(char *datafile, int emaxnumofcurves, int maxm, std::string ccdir) {
 
   Rprintf("Reading %s\n", datafile);
 	char line[BUFFSZ];
-	char key[10];
+	char key[20];
 	int i=0, nm, j;
 
 
@@ -181,6 +181,30 @@ Input::Input(char *datafile, int emaxnumofcurves, int maxm, std::string ccdir) {
 			continue;
 		}
 
+        if (strcmp( key, "DetCensor") == 0){
+			sscanf( pars[0], " %s", line);
+                        //Censored rc measurement, result Unknown ABOVE y
+                        //with expected std 
+					   //DetCensor(char *enm, double ey, double estd, double x, double edeltaR, double edeltaSTD, double ea, double eb, Cal *ecc)
+			tmpdet = new DetCensor(  line   ,  rpars[1],    rpars[2], rpars[3],       rpars[4],         rpars[5],  rpars[6],  rpars[7], curves[(int) rpars[8]]);
+
+			dets->AddDet(tmpdet);
+
+			continue;
+		}
+
+        if (strcmp( key, "DetCensorE") == 0){
+			sscanf( pars[0], " %s", line);
+                        //Censored rc measurement, result Unknown BELOW y
+                        //with expected std 
+					   //DetCensor(char *enm, double ey, double estd, double x, double edeltaR, double edeltaSTD, double ea, double eb, Cal *ecc)
+			tmpdet = new DetCensorE(  line   ,  rpars[1],    rpars[2], rpars[3],       rpars[4],         rpars[5],  rpars[6],  rpars[7], curves[(int) rpars[8]]);
+
+			dets->AddDet(tmpdet);
+
+			continue;
+		}
+		
 
 		if (strcmp( key, "Hiatus") == 0)
 		{
@@ -233,10 +257,47 @@ Input::Input(char *datafile, int emaxnumofcurves, int maxm, std::string ccdir) {
 					rpars[2], rpars[3], rpars[4], rpars[5], rpars[10], rpars[11], 1,      seed, more_pars);
 					//MinYr     MaxYr       th0      thp0         c0       cm  useT
 			}
-
-
-			//Then open the twalk object
-			BaconTwalk = new twalk( *bacon, bacon->Getx0(), bacon->Getxp0(), bacon->get_dim());
+            
+            strcpy( init_v_fnam, datafile);
+            strcat( init_v_fnam, ".init");//File name for initial values
+            strcpy( last_v_fnam, datafile);
+            strcat( last_v_fnam, ".last");//File name for last values
+            bacon_dim = bacon->get_dim();
+            if ((IV = fopen( init_v_fnam, "r")) == NULL){
+                Rprintf("Since no file %s was provided with initial values for the twalk, I will be using simulated values.\n", init_v_fnam);
+                X0 = bacon->Getx0();
+                Xp0 = bacon->Getxp0(); }
+            else { //Read initail values into X0 and Xp0
+                Rprintf("Initial values for the twalk taken from %s.\n", init_v_fnam);
+                X0 = bacon->Getx0();
+                Xp0 = bacon->Getxp0();
+                for (int k=0; k<bacon_dim; k++) {
+                    if (fscanf( IV, " %lf", X0+k) == 0){
+                        Rprintf("File %s with initial values, incomplete.\n", init_v_fnam);
+                        Rcpp::stop("File %s with initial values, incomplete.\n", init_v_fnam);
+                    }
+                    //Rprintf("%g ", X0[k]);
+                }
+                //Rprintf("\n");
+                for (int k=0; k<bacon_dim; k++) {
+                    if (fscanf( IV, " %lf", Xp0+k) == 0){
+                        Rprintf("File %s with initial values, incomplete.\n", init_v_fnam);
+                        Rcpp::stop("File %s with initial values, incomplete.\n", init_v_fnam);
+                    }
+                    //Rprintf("%g ", Xp0[k]);
+                }
+                //Rprintf("\n");
+                fclose(IV);
+            }
+            //Now open .last for writing, append
+            if ((LV = fopen( last_v_fnam, "a+")) == NULL){
+                Rprintf("Could not open %s for writing.\n", last_v_fnam);
+                Rcpp::stop("Could not open %s for reading\n", last_v_fnam); 
+            }
+            //Here, leave the pointer at the end of the file
+            //fseek( LV, 0, SEEK_END);
+            //Then open the twalk object
+			BaconTwalk = new twalk( *bacon, X0, Xp0, bacon->get_dim());
 
 			break;  //this should be the last key in the program
 		}
